@@ -35,6 +35,10 @@ def _format_admin_state(state):
     return 'UP' if state else 'DOWN'
 
 
+def _format_dns_assignment(dns_assignment):
+    return utils.format_list_of_dicts(dns_assignment) \
+        if dns_assignment else None
+
 _formatters = {
     'admin_state_up': _format_admin_state,
     'is_admin_state_up': _format_admin_state,
@@ -43,7 +47,7 @@ _formatters = {
     'binding_vif_details': utils.format_dict,
     'binding:profile': utils.format_dict,
     'binding:vif_details': utils.format_dict,
-    'dns_assignment': utils.format_list_of_dicts,
+    'dns_assignment': _format_dns_assignment,
     'extra_dhcp_opts': utils.format_list_of_dicts,
     'fixed_ips': utils.format_list_of_dicts,
     'security_group_ids': utils.format_list,
@@ -246,6 +250,17 @@ def _add_updatable_args(parser):
 
 # TODO(abhiraut): Use the SDK resource mapped attribute names once the
 # OSC minimum requirements include SDK 1.0.
+def _convert_address_pairs(parsed_args):
+    ops = []
+    for opt in parsed_args.allowed_address_pairs:
+        addr = {}
+        addr['ip_address'] = opt['ip-address']
+        if 'mac-address' in opt:
+            addr['mac_address'] = opt['mac-address']
+        ops.append(addr)
+    return ops
+
+
 class CreatePort(command.ShowOne):
     _description = _("Create a new port")
 
@@ -305,7 +320,7 @@ class CreatePort(command.ShowOne):
             help=_("Name of this port")
         )
         # TODO(singhj): Add support for extended options:
-        # qos,dhcp, address pairs
+        # qos,dhcp
         secgroups = parser.add_mutually_exclusive_group()
         secgroups.add_argument(
             '--security-group',
@@ -332,7 +347,17 @@ class CreatePort(command.ShowOne):
             action='store_true',
             help=_("Disable port security for this port")
         )
-
+        parser.add_argument(
+            '--allowed-address',
+            metavar='ip-address=<ip-address>[,mac-address=<mac-address>]',
+            action=parseractions.MultiKeyValueAction,
+            dest='allowed_address_pairs',
+            required_keys=['ip-address'],
+            optional_keys=['mac-address'],
+            help=_("Add allowed-address pair associated with this port: "
+                   "ip-address=<ip-address>[,mac-address=<mac-address>] "
+                   "(repeat option to set multiple allowed-address pairs)")
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -349,6 +374,9 @@ class CreatePort(command.ShowOne):
                                         for sg in parsed_args.security_groups]
         if parsed_args.no_security_group:
             attrs['security_groups'] = []
+        if parsed_args.allowed_address_pairs:
+            attrs['allowed_address_pairs'] = (
+                _convert_address_pairs(parsed_args))
 
         obj = client.create_port(**attrs)
         display_columns, columns = _get_columns(obj)
@@ -432,11 +460,18 @@ class ListPort(command.Lister):
             default=False,
             help=_("List additional fields in output")
         )
+        parser.add_argument(
+            '--project',
+            metavar='<project>',
+            help=_("List ports according to their project (name or ID)")
+        )
+        identity_common.add_project_domain_option_to_parser(parser)
         return parser
 
     def take_action(self, parsed_args):
         network_client = self.app.client_manager.network
         compute_client = self.app.client_manager.compute
+        identity_client = self.app.client_manager.identity
 
         columns = (
             'id',
@@ -473,6 +508,14 @@ class ListPort(command.Lister):
             filters['network_id'] = network.id
         if parsed_args.mac_address:
             filters['mac_address'] = parsed_args.mac_address
+        if parsed_args.project:
+            project_id = identity_common.find_project(
+                identity_client,
+                parsed_args.project,
+                parsed_args.project_domain,
+            ).id
+            filters['tenant_id'] = project_id
+            filters['project_id'] = project_id
 
         data = network_client.ports(**filters)
 
@@ -569,7 +612,26 @@ class SetPort(command.Command):
             action='store_true',
             help=_("Disable port security for this port")
         )
-
+        parser.add_argument(
+            '--allowed-address',
+            metavar='ip-address=<ip-address>[,mac-address=<mac-address>]',
+            action=parseractions.MultiKeyValueAction,
+            dest='allowed_address_pairs',
+            required_keys=['ip-address'],
+            optional_keys=['mac-address'],
+            help=_("Add allowed-address pair associated with this port: "
+                   "ip-address=<ip-address>[,mac-address=<mac-address>] "
+                   "(repeat option to set multiple allowed-address pairs)")
+        )
+        parser.add_argument(
+            '--no-allowed-address',
+            dest='no_allowed_address_pair',
+            action='store_true',
+            help=_("Clear existing allowed-address pairs associated"
+                   "with this port."
+                   "(Specify both --allowed-address and --no-allowed-address"
+                   "to overwrite the current allowed-address pairs)")
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -609,6 +671,19 @@ class SetPort(command.Command):
         elif parsed_args.no_security_group:
             attrs['security_groups'] = []
 
+        if (parsed_args.allowed_address_pairs and
+                parsed_args.no_allowed_address_pair):
+            attrs['allowed_address_pairs'] = (
+                _convert_address_pairs(parsed_args))
+
+        elif parsed_args.allowed_address_pairs:
+            attrs['allowed_address_pairs'] = (
+                [addr for addr in obj.allowed_address_pairs if addr] +
+                _convert_address_pairs(parsed_args))
+
+        elif parsed_args.no_allowed_address_pair:
+            attrs['allowed_address_pairs'] = []
+
         client.update_port(obj, **attrs)
 
 
@@ -669,6 +744,19 @@ class UnsetPort(command.Command):
             metavar="<port>",
             help=_("Port to modify (name or ID)")
         )
+        parser.add_argument(
+            '--allowed-address',
+            metavar='ip-address=<ip-address>[,mac-address=<mac-address>]',
+            action=parseractions.MultiKeyValueAction,
+            dest='allowed_address_pairs',
+            required_keys=['ip-address'],
+            optional_keys=['mac-address'],
+            help=_("Desired allowed-address pair which should be removed "
+                   "from this port: ip-address=<ip-address> "
+                   "[,mac-address=<mac-address>] (repeat option to set "
+                   "multiple allowed-address pairs)")
+        )
+
         return parser
 
     def take_action(self, parsed_args):
@@ -680,6 +768,7 @@ class UnsetPort(command.Command):
         tmp_fixed_ips = copy.deepcopy(obj.fixed_ips)
         tmp_binding_profile = copy.deepcopy(obj.binding_profile)
         tmp_secgroups = copy.deepcopy(obj.security_groups)
+        tmp_addr_pairs = copy.deepcopy(obj.allowed_address_pairs)
         _prepare_fixed_ips(self.app.client_manager, parsed_args)
         attrs = {}
         if parsed_args.fixed_ip:
@@ -708,6 +797,14 @@ class UnsetPort(command.Command):
                 msg = _("Port does not contain security group %s") % sg
                 raise exceptions.CommandError(msg)
             attrs['security_groups'] = tmp_secgroups
+        if parsed_args.allowed_address_pairs:
+            try:
+                for addr in _convert_address_pairs(parsed_args):
+                    tmp_addr_pairs.remove(addr)
+            except ValueError:
+                msg = _("Port does not contain allowed-address-pair %s") % addr
+                raise exceptions.CommandError(msg)
+            attrs['allowed_address_pairs'] = tmp_addr_pairs
 
         if attrs:
             client.update_port(obj, **attrs)
