@@ -16,6 +16,7 @@
 import mock
 
 from osc_lib import exceptions
+from osc_lib import utils
 
 from openstackclient.identity.v3 import project
 from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes
@@ -445,6 +446,32 @@ class TestProjectDelete(TestProject):
         )
         self.assertIsNone(result)
 
+    @mock.patch.object(utils, 'find_resource')
+    def test_delete_multi_projects_with_exception(self, find_mock):
+        find_mock.side_effect = [self.project,
+                                 exceptions.CommandError]
+        arglist = [
+            self.project.id,
+            'unexist_project',
+        ]
+        verifylist = [
+            ('projects', arglist),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        try:
+            self.cmd.take_action(parsed_args)
+            self.fail('CommandError should be raised.')
+        except exceptions.CommandError as e:
+            self.assertEqual('1 of 2 projects failed to delete.',
+                             str(e))
+
+        find_mock.assert_any_call(self.projects_mock, self.project.id)
+        find_mock.assert_any_call(self.projects_mock, 'unexist_project')
+
+        self.assertEqual(2, find_mock.call_count)
+        self.projects_mock.delete.assert_called_once_with(self.project.id)
+
 
 class TestProjectList(TestProject):
 
