@@ -46,6 +46,9 @@ class TestVolume(volume_fakes.TestVolume):
         self.snapshots_mock = self.app.client_manager.volume.volume_snapshots
         self.snapshots_mock.reset_mock()
 
+        self.types_mock = self.app.client_manager.volume.volume_types
+        self.types_mock.reset_mock()
+
         self.consistencygroups_mock = (
             self.app.client_manager.volume.consistencygroups)
         self.consistencygroups_mock.reset_mock()
@@ -447,6 +450,154 @@ class TestVolumeCreate(TestVolume):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, data)
 
+    def test_volume_create_with_bootable_and_readonly(self):
+        arglist = [
+            '--bootable',
+            '--read-only',
+            '--size', str(self.new_volume.size),
+            self.new_volume.name,
+        ]
+        verifylist = [
+            ('bootable', True),
+            ('non_bootable', False),
+            ('read_only', True),
+            ('read_write', False),
+            ('size', self.new_volume.size),
+            ('name', self.new_volume.name),
+        ]
+
+        parsed_args = self.check_parser(
+            self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.create.assert_called_with(
+            size=self.new_volume.size,
+            snapshot_id=None,
+            name=self.new_volume.name,
+            description=None,
+            volume_type=None,
+            user_id=None,
+            project_id=None,
+            availability_zone=None,
+            metadata=None,
+            imageRef=None,
+            source_volid=None,
+            consistencygroup_id=None,
+            source_replica=None,
+            multiattach=False,
+            scheduler_hints=None,
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
+        self.volumes_mock.set_bootable.assert_called_with(
+            self.new_volume.id, True)
+        self.volumes_mock.update_readonly_flag.assert_called_with(
+            self.new_volume.id, True)
+
+    def test_volume_create_with_nonbootable_and_readwrite(self):
+        arglist = [
+            '--non-bootable',
+            '--read-write',
+            '--size', str(self.new_volume.size),
+            self.new_volume.name,
+        ]
+        verifylist = [
+            ('bootable', False),
+            ('non_bootable', True),
+            ('read_only', False),
+            ('read_write', True),
+            ('size', self.new_volume.size),
+            ('name', self.new_volume.name),
+        ]
+
+        parsed_args = self.check_parser(
+            self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.create.assert_called_with(
+            size=self.new_volume.size,
+            snapshot_id=None,
+            name=self.new_volume.name,
+            description=None,
+            volume_type=None,
+            user_id=None,
+            project_id=None,
+            availability_zone=None,
+            metadata=None,
+            imageRef=None,
+            source_volid=None,
+            consistencygroup_id=None,
+            source_replica=None,
+            multiattach=False,
+            scheduler_hints=None,
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
+        self.volumes_mock.set_bootable.assert_called_with(
+            self.new_volume.id, False)
+        self.volumes_mock.update_readonly_flag.assert_called_with(
+            self.new_volume.id, False)
+
+    @mock.patch.object(volume.LOG, 'error')
+    def test_volume_create_with_bootable_and_readonly_fail(
+            self, mock_error):
+
+        self.volumes_mock.set_bootable.side_effect = (
+            exceptions.CommandError())
+
+        self.volumes_mock.update_readonly_flag.side_effect = (
+            exceptions.CommandError())
+
+        arglist = [
+            '--bootable',
+            '--read-only',
+            '--size', str(self.new_volume.size),
+            self.new_volume.name,
+        ]
+        verifylist = [
+            ('bootable', True),
+            ('non_bootable', False),
+            ('read_only', True),
+            ('read_write', False),
+            ('size', self.new_volume.size),
+            ('name', self.new_volume.name),
+        ]
+
+        parsed_args = self.check_parser(
+            self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.create.assert_called_with(
+            size=self.new_volume.size,
+            snapshot_id=None,
+            name=self.new_volume.name,
+            description=None,
+            volume_type=None,
+            user_id=None,
+            project_id=None,
+            availability_zone=None,
+            metadata=None,
+            imageRef=None,
+            source_volid=None,
+            consistencygroup_id=None,
+            source_replica=None,
+            multiattach=False,
+            scheduler_hints=None,
+        )
+
+        self.assertEqual(2, mock_error.call_count)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
+        self.volumes_mock.set_bootable.assert_called_with(
+            self.new_volume.id, True)
+        self.volumes_mock.update_readonly_flag.assert_called_with(
+            self.new_volume.id, True)
+
     def test_volume_create_with_source_replicated(self):
         self.volumes_mock.get.return_value = self.new_volume
         arglist = [
@@ -672,6 +823,19 @@ class TestVolumeList(TestVolume):
 
         columns, data = self.cmd.take_action(parsed_args)
 
+        search_opts = {
+            'all_tenants': False,
+            'project_id': None,
+            'user_id': None,
+            'display_name': None,
+            'status': None,
+        }
+        self.volumes_mock.list.assert_called_once_with(
+            search_opts=search_opts,
+            marker=None,
+            limit=None,
+        )
+
         self.assertEqual(self.columns, columns)
 
         server = self.mock_volume.attachments[0]['server_id']
@@ -702,6 +866,19 @@ class TestVolumeList(TestVolume):
 
         columns, data = self.cmd.take_action(parsed_args)
 
+        search_opts = {
+            'all_tenants': True,
+            'project_id': self.project.id,
+            'user_id': None,
+            'display_name': None,
+            'status': None,
+        }
+        self.volumes_mock.list.assert_called_once_with(
+            search_opts=search_opts,
+            marker=None,
+            limit=None,
+        )
+
         self.assertEqual(self.columns, columns)
 
         server = self.mock_volume.attachments[0]['server_id']
@@ -734,6 +911,19 @@ class TestVolumeList(TestVolume):
 
         columns, data = self.cmd.take_action(parsed_args)
 
+        search_opts = {
+            'all_tenants': True,
+            'project_id': self.project.id,
+            'user_id': None,
+            'display_name': None,
+            'status': None,
+        }
+        self.volumes_mock.list.assert_called_once_with(
+            search_opts=search_opts,
+            marker=None,
+            limit=None,
+        )
+
         self.assertEqual(self.columns, columns)
 
         server = self.mock_volume.attachments[0]['server_id']
@@ -764,6 +954,19 @@ class TestVolumeList(TestVolume):
 
         columns, data = self.cmd.take_action(parsed_args)
 
+        search_opts = {
+            'all_tenants': False,
+            'project_id': None,
+            'user_id': self.user.id,
+            'display_name': None,
+            'status': None,
+        }
+        self.volumes_mock.list.assert_called_once_with(
+            search_opts=search_opts,
+            marker=None,
+            limit=None,
+        )
+
         self.assertEqual(self.columns, columns)
         server = self.mock_volume.attachments[0]['server_id']
         device = self.mock_volume.attachments[0]['device']
@@ -795,6 +998,19 @@ class TestVolumeList(TestVolume):
 
         columns, data = self.cmd.take_action(parsed_args)
 
+        search_opts = {
+            'all_tenants': False,
+            'project_id': None,
+            'user_id': self.user.id,
+            'display_name': None,
+            'status': None,
+        }
+        self.volumes_mock.list.assert_called_once_with(
+            search_opts=search_opts,
+            marker=None,
+            limit=None,
+        )
+
         self.assertEqual(self.columns, columns)
 
         server = self.mock_volume.attachments[0]['server_id']
@@ -825,6 +1041,19 @@ class TestVolumeList(TestVolume):
 
         columns, data = self.cmd.take_action(parsed_args)
 
+        search_opts = {
+            'all_tenants': False,
+            'project_id': None,
+            'user_id': None,
+            'display_name': self.mock_volume.name,
+            'status': None,
+        }
+        self.volumes_mock.list.assert_called_once_with(
+            search_opts=search_opts,
+            marker=None,
+            limit=None,
+        )
+
         self.assertEqual(self.columns, columns)
 
         server = self.mock_volume.attachments[0]['server_id']
@@ -855,6 +1084,19 @@ class TestVolumeList(TestVolume):
 
         columns, data = self.cmd.take_action(parsed_args)
 
+        search_opts = {
+            'all_tenants': False,
+            'project_id': None,
+            'user_id': None,
+            'display_name': None,
+            'status': self.mock_volume.status,
+        }
+        self.volumes_mock.list.assert_called_once_with(
+            search_opts=search_opts,
+            marker=None,
+            limit=None,
+        )
+
         self.assertEqual(self.columns, columns)
 
         server = self.mock_volume.attachments[0]['server_id']
@@ -885,6 +1127,19 @@ class TestVolumeList(TestVolume):
 
         columns, data = self.cmd.take_action(parsed_args)
 
+        search_opts = {
+            'all_tenants': True,
+            'project_id': None,
+            'user_id': None,
+            'display_name': None,
+            'status': None,
+        }
+        self.volumes_mock.list.assert_called_once_with(
+            search_opts=search_opts,
+            marker=None,
+            limit=None,
+        )
+
         self.assertEqual(self.columns, columns)
 
         server = self.mock_volume.attachments[0]['server_id']
@@ -916,6 +1171,19 @@ class TestVolumeList(TestVolume):
 
         columns, data = self.cmd.take_action(parsed_args)
 
+        search_opts = {
+            'all_tenants': False,
+            'project_id': None,
+            'user_id': None,
+            'display_name': None,
+            'status': None,
+        }
+        self.volumes_mock.list.assert_called_once_with(
+            search_opts=search_opts,
+            marker=None,
+            limit=None,
+        )
+
         collist = [
             'ID',
             'Display Name',
@@ -1088,11 +1356,14 @@ class TestVolumeMigrate(TestVolume):
 
 class TestVolumeSet(TestVolume):
 
+    volume_type = volume_fakes.FakeType.create_one_type()
+
     def setUp(self):
         super(TestVolumeSet, self).setUp()
 
         self.new_volume = volume_fakes.FakeVolume.create_one_volume()
         self.volumes_mock.get.return_value = self.new_volume
+        self.types_mock.get.return_value = self.volume_type
 
         # Get the command object to test
         self.cmd = volume.SetVolume(self.app, None)
@@ -1221,6 +1492,66 @@ class TestVolumeSet(TestVolume):
             False)
         self.assertIsNone(result)
 
+    def test_volume_set_type(self):
+        arglist = [
+            '--type', self.volume_type.id,
+            self.new_volume.id
+        ]
+        verifylist = [
+            ('retype_policy', None),
+            ('type', self.volume_type.id),
+            ('volume', self.new_volume.id)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.retype.assert_called_once_with(
+            self.new_volume.id,
+            self.volume_type.id,
+            'never')
+        self.assertIsNone(result)
+
+    def test_volume_set_type_with_policy(self):
+        arglist = [
+            '--retype-policy', 'on-demand',
+            '--type', self.volume_type.id,
+            self.new_volume.id
+        ]
+        verifylist = [
+            ('retype_policy', 'on-demand'),
+            ('type', self.volume_type.id),
+            ('volume', self.new_volume.id)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.retype.assert_called_once_with(
+            self.new_volume.id,
+            self.volume_type.id,
+            'on-demand')
+        self.assertIsNone(result)
+
+    @mock.patch.object(volume.LOG, 'warning')
+    def test_volume_set_with_only_retype_policy(self, mock_warning):
+        arglist = [
+            '--retype-policy', 'on-demand',
+            self.new_volume.id
+        ]
+        verifylist = [
+            ('retype_policy', 'on-demand'),
+            ('volume', self.new_volume.id)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.retype.assert_not_called()
+        mock_warning.assert_called_with("'--retype-policy' option will "
+                                        "not work without '--type' option")
+        self.assertIsNone(result)
+
 
 class TestVolumeShow(TestVolume):
 
