@@ -132,6 +132,28 @@ class CreateVolume(command.ShowOne):
             help=_("Allow volume to be attached more than once "
                    "(default to False)")
         )
+        bootable_group = parser.add_mutually_exclusive_group()
+        bootable_group.add_argument(
+            "--bootable",
+            action="store_true",
+            help=_("Mark volume as bootable")
+        )
+        bootable_group.add_argument(
+            "--non-bootable",
+            action="store_true",
+            help=_("Mark volume as non-bootable (default)")
+        )
+        readonly_group = parser.add_mutually_exclusive_group()
+        readonly_group.add_argument(
+            "--read-only",
+            action="store_true",
+            help=_("Set volume to read-only access mode")
+        )
+        readonly_group.add_argument(
+            "--read-write",
+            action="store_true",
+            help=_("Set volume to read-write access mode (default)")
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -199,6 +221,22 @@ class CreateVolume(command.ShowOne):
             multiattach=parsed_args.multi_attach,
             scheduler_hints=parsed_args.hint,
         )
+
+        if parsed_args.bootable or parsed_args.non_bootable:
+            try:
+                volume_client.volumes.set_bootable(
+                    volume.id, parsed_args.bootable)
+            except Exception as e:
+                LOG.error(_("Failed to set volume bootable property: %s"), e)
+        if parsed_args.read_only or parsed_args.read_write:
+            try:
+                volume_client.volumes.update_readonly_flag(
+                    volume.id,
+                    parsed_args.read_only)
+            except Exception as e:
+                LOG.error(_("Failed to set volume read-only access "
+                            "mode flag: %s"), e)
+
         # Remove key links from being displayed
         volume._info.update(
             {
@@ -379,16 +417,19 @@ class ListVolume(command.Lister):
             project_id = identity_common.find_project(
                 identity_client,
                 parsed_args.project,
-                parsed_args.project_domain)
+                parsed_args.project_domain).id
 
         user_id = None
         if parsed_args.user:
             user_id = identity_common.find_user(identity_client,
                                                 parsed_args.user,
-                                                parsed_args.user_domain)
+                                                parsed_args.user_domain).id
+
+        # set value of 'all_tenants' when using project option
+        all_projects = bool(parsed_args.project) or parsed_args.all_projects
 
         search_opts = {
-            'all_tenants': parsed_args.all_projects,
+            'all_tenants': all_projects,
             'project_id': project_id,
             'user_id': user_id,
             'display_name': parsed_args.name,
@@ -509,6 +550,19 @@ class SetVolume(command.Command):
                    'in the database with no regard to actual status, '
                    'exercise caution when using)'),
         )
+        parser.add_argument(
+            '--type',
+            metavar='<volume-type>',
+            help=_('New volume type (name or ID)'),
+        )
+        parser.add_argument(
+            '--retype-policy',
+            metavar='<retype-policy>',
+            choices=['never', 'on-demand'],
+            help=_('Migration policy while re-typing volume '
+                   '("never" or "on-demand", default is "never" ) '
+                   '(available only when "--type" option is specified)'),
+        )
         bootable_group = parser.add_mutually_exclusive_group()
         bootable_group.add_argument(
             "--bootable",
@@ -590,6 +644,28 @@ class SetVolume(command.Command):
                 LOG.error(_("Failed to set volume read-only access "
                             "mode flag: %s"), e)
                 result += 1
+        if parsed_args.type:
+            # get the migration policy
+            migration_policy = 'never'
+            if parsed_args.retype_policy:
+                migration_policy = parsed_args.retype_policy
+            try:
+                # find the volume type
+                volume_type = utils.find_resource(
+                    volume_client.volume_types,
+                    parsed_args.type)
+                # reset to the new volume type
+                volume_client.volumes.retype(
+                    volume.id,
+                    volume_type.id,
+                    migration_policy)
+            except Exception as e:
+                LOG.error(_("Failed to set volume type: %s"), e)
+                result += 1
+        elif parsed_args.retype_policy:
+            # If the "--retype-policy" is specified without "--type"
+            LOG.warning(_("'--retype-policy' option will not work "
+                          "without '--type' option"))
 
         kwargs = {}
         if parsed_args.name:
